/////////////////////////////////////////////////////////////////////////
// File:      taskbar.cpp
// Purpose:	  Implements CTaskBarIcon class for manipulating icons on
//            task bar.
// Author:    Julian Smart
// Created:   24/3/98
/////////////////////////////////////////////////////////////////////////

/*#include <string.h>
#include <windows.h>
#include <shellapi.h>
*/
#include "stdafx.h"
#include "taskbar.h"
#include "resource.h"

#define _EXPORT __declspec(dllexport)

LONG _EXPORT FAR PASCAL CTaskBarIconWindowProc( HWND hWnd, unsigned msg,
				     UINT wParam, LONG lParam );

char *CTaskBarWindowClass = "CTaskBarWindowClass";

CTaskBarIcon* CTaskBarIcon::sm_first = NULL;
BOOL   CTaskBarIcon::sm_registeredClass = FALSE;
UINT   CTaskBarIcon::sm_taskbarMsg = 0;

CTaskBarIcon::CTaskBarIcon(void)
{
    m_hWnd = 0;
    m_iconAdded = FALSE;
    m_next = NULL;

    AddObject(this);

    if (RegisterWindowClass())
        m_hWnd = CreateTaskBarWindow();
}

CTaskBarIcon::~CTaskBarIcon(void)
{
    RemoveObject(this);

    if (m_iconAdded)
    {
        RemoveIcon();
    }

    if (m_hWnd)
    {
        ::DestroyWindow(m_hWnd);
        m_hWnd = 0;
    }
}

// Operations
BOOL CTaskBarIcon::SetIcon(HICON hIcon, const char* tooltip)
{
    if (!IsOK())
        return FALSE;

    NOTIFYICONDATA notifyData;

    memset(&notifyData, 0, sizeof(notifyData));
	notifyData.cbSize = sizeof(notifyData);
	notifyData.hWnd = (HWND) m_hWnd;
	notifyData.uCallbackMessage = sm_taskbarMsg;
	notifyData.uFlags = NIF_MESSAGE ;
	if (hIcon != 0)
		notifyData.uFlags |= NIF_ICON;

    if (((const char*) tooltip != NULL) && (tooltip != ""))
    {
        notifyData.uFlags |= NIF_TIP ;
		lstrcpyn(notifyData.szTip, (char*) tooltip, sizeof(notifyData.szTip));
    }

	notifyData.hIcon = (HICON) hIcon;
    notifyData.uID = 99;

    if (m_iconAdded)
        return Shell_NotifyIcon(NIM_MODIFY, & notifyData);
    else
    {
        m_iconAdded = Shell_NotifyIcon(NIM_ADD, & notifyData);
        return m_iconAdded;
    }
}

BOOL CTaskBarIcon::RemoveIcon(void)
{
    if (!m_iconAdded)
        return FALSE;

    NOTIFYICONDATA notifyData;

    memset(&notifyData, 0, sizeof(notifyData));
	notifyData.cbSize = sizeof(notifyData);
	notifyData.hWnd = (HWND) m_hWnd;
	notifyData.uCallbackMessage = sm_taskbarMsg;
	notifyData.uFlags = NIF_MESSAGE;
	notifyData.hIcon = 0 ; // hIcon;
    notifyData.uID = 99;
    m_iconAdded = FALSE;

    return Shell_NotifyIcon(NIM_DELETE, & notifyData);
}

// Overridables
void CTaskBarIcon::OnMouseMove(void)
{
}

void CTaskBarIcon::OnLButtonDown(void)
{
}

void CTaskBarIcon::OnLButtonUp(void)
{
}

void CTaskBarIcon::OnRButtonDown(void)
{
}

void CTaskBarIcon::OnRButtonUp(void)
{
}

void CTaskBarIcon::OnLButtonDClick(void)
{
}

void CTaskBarIcon::OnRButtonDClick(void)
{
}

CTaskBarIcon* CTaskBarIcon::FindObjectForHWND(HWND hWnd)
{
    CTaskBarIcon* obj = sm_first;
    while (obj)
    {
        if (obj->GetHWND() == hWnd)
            return obj;
        obj = obj->m_next;
    }
    return NULL;
}

void CTaskBarIcon::AddObject(CTaskBarIcon* obj)
{
    if (sm_first == NULL)
        sm_first = obj;
    else
    {
        CTaskBarIcon* o = sm_first;
        while (o)
        {
            if (o->m_next == NULL)
            {
                o->m_next = obj;
                return;
            }
            else
                o = o->m_next;
        }
    }
}

void CTaskBarIcon::RemoveObject(CTaskBarIcon* obj)
{
    if (sm_first == obj)
    {
        sm_first = sm_first->m_next;
    }
    else
    {
        CTaskBarIcon* o = sm_first;
        while (o)
        {
            if (o->m_next == obj)
            {
                o->m_next = obj->m_next;
                return;
            }
            o = o->m_next;
        }
    }
}

BOOL CTaskBarIcon::RegisterWindowClass()
{
    if (sm_registeredClass)
        return TRUE;

    // Also register the taskbar message here
    sm_taskbarMsg = ::RegisterWindowMessage("CtaskBarIconMessage");

    WNDCLASS	wc;
    BOOL	rc;

    HINSTANCE hInstance = GetModuleHandle(NULL);

    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) CTaskBarIconWindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = 0;
    wc.hCursor = 0;
    wc.hbrBackground = 0;
    wc.lpszMenuName = NULL;
    wc.lpszClassName = CTaskBarWindowClass ;
    rc = ::RegisterClass( &wc );

    sm_registeredClass = (rc != 0);

    return( (rc != 0) );
}

HWND CTaskBarIcon::CreateTaskBarWindow()
{
    HINSTANCE hInstance = GetModuleHandle(NULL);

    HWND hWnd = CreateWindowEx (0, CTaskBarWindowClass,
            "CTaskBarWindow",
            WS_OVERLAPPED,
            0,
            0,
            10,
            10,
            NULL,
            (HMENU) 0,
            hInstance,
            NULL);

    DWORD err = GetLastError();

    return hWnd;
}

long CTaskBarIcon::WindowProc( HWND hWnd, unsigned int msg, unsigned int wParam, long lParam )
{
    if (msg != sm_taskbarMsg)
        return DefWindowProc(hWnd, msg, wParam, lParam);

    switch (lParam)
    {
	case WM_LBUTTONDOWN:
		OnLButtonDown();
		break;

	case WM_LBUTTONUP:
		OnLButtonUp();
		break;

	case WM_RBUTTONDOWN:
		OnRButtonDown();
		break;

	case WM_RBUTTONUP:
		OnRButtonUp();
		break;

	case WM_LBUTTONDBLCLK:
		OnLButtonDClick();
		break;

	case WM_RBUTTONDBLCLK:
		OnRButtonDClick();
		break;

	case WM_MOUSEMOVE:
		OnMouseMove();
		break;

	default:
        break;
	}
    return 0;
}

LONG _EXPORT FAR PASCAL CTaskBarIconWindowProc( HWND hWnd, unsigned msg,
				     UINT wParam, LONG lParam )
{
    CTaskBarIcon* obj = CTaskBarIcon::FindObjectForHWND(hWnd);
    if (obj)
        return obj->WindowProc(hWnd, msg, wParam, lParam);
    else
        return DefWindowProc(hWnd, msg, wParam, lParam);
}
