/////////////////////////////////////////////////////////////////////////
// File:      taskbar.h
// Purpose:	  Defines CTaskBarIcon class for manipulating icons on task bar.
// Author:    Julian Smart
// Created:   24/3/98
/////////////////////////////////////////////////////////////////////////

#ifndef _TASKBAR_H_
#define _TASKBAR_H_

class CTaskBarIcon
{
public:
	CTaskBarIcon(void);
	virtual ~CTaskBarIcon(void);

// Accessors
    inline HWND GetHWND() const { return m_hWnd; }
    inline BOOL IsOK() const { return (m_hWnd != 0) ; }
    inline BOOL IsIconInstalled() const { return m_iconAdded; }

// Operations
    BOOL SetIcon(HICON hIcon, const char* tooltip = "");
    BOOL RemoveIcon(void);

// Overridables
    virtual void OnMouseMove(void);
    virtual void OnLButtonDown(void);
    virtual void OnLButtonUp(void);
    virtual void OnRButtonDown(void);
    virtual void OnRButtonUp(void);
    virtual void OnLButtonDClick(void);
    virtual void OnRButtonDClick(void);

// Implementation
public:
    long WindowProc( HWND hWnd, unsigned int msg, unsigned int wParam, long lParam );
    static CTaskBarIcon* FindObjectForHWND(HWND hWnd);
protected:
    static void AddObject(CTaskBarIcon* obj);
    static void RemoveObject(CTaskBarIcon* obj);
    static BOOL RegisterWindowClass();
    static HWND CreateTaskBarWindow();

// Data members
protected:
    HWND                    m_hWnd;
    BOOL                    m_iconAdded;
    static BOOL             sm_registeredClass;
    static unsigned int     sm_taskbarMsg;
    static CTaskBarIcon*    sm_first;
    CTaskBarIcon*           m_next;
};

#endif
    // _TASKBAR_H_

